package fr.nitorac.aurionweb;

import android.content.Context;
import android.graphics.Point;
import android.util.Log;
import android.view.Display;
import android.view.WindowManager;

import com.google.gson.JsonObject;

import org.jsoup.Connection;
import org.jsoup.Jsoup;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;
import java.security.GeneralSecurityException;
import java.security.KeyStore;
import java.security.cert.Certificate;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.text.CharacterIterator;
import java.text.Normalizer;
import java.text.StringCharacterIterator;
import java.util.Locale;

import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManagerFactory;

public class Utils {

    public final static String BASE_SSL_URL = "https://ranini.iiens.net/aurionweb-check/cert.php?site=";
    private static SSLContext sslContext;
    private static Certificate casCert;
    private static Certificate aurionCert;

    public static Connection wrapSSL(String url) throws GeneralSecurityException, IOException {
        return Jsoup.connect(url)
                .sslSocketFactory(getContextForTrustedCertificates(MainActivity.getInstance().getApplicationContext()).getSocketFactory());
    }

    public static CertificateFactory getSSLFactory() throws CertificateException {
        return CertificateFactory.getInstance("X.509");
    }

    public static Certificate getCasCert(Context c) throws CertificateException {
        if (casCert == null){
            InputStream input;
            try{
                URLConnection conn = new URL(BASE_SSL_URL + "cas").openConnection();
                conn.connect();
                input = new BufferedInputStream(conn.getInputStream());
            }catch (IOException e){
                e.printStackTrace();
                Log.e("SSLCertFetch", "Can't get remote SSL cas certificate, fallback on local one");
                input = c.getResources().openRawResource(R.raw.cas_ensiie_fr);
            }
            casCert = getSSLFactory().generateCertificate(input);
        }
        return casCert;
    }

    public static Certificate getAurionCert(Context c) throws CertificateException {
        if (aurionCert == null){
            InputStream input;
            try{
                URLConnection conn = new URL(BASE_SSL_URL + "aurion").openConnection();
                conn.connect();
                input = new BufferedInputStream(conn.getInputStream());
            }catch (IOException e){
                e.printStackTrace();
                Log.e("SSLCertFetch", "Can't get remote SSL aurion certificate, fallback on local one");
                input = c.getResources().openRawResource(R.raw.aurionweb_ensiie_fr);
            }
            aurionCert = getSSLFactory().generateCertificate(input);
        }
        return aurionCert;
    }

    // Comme les certificats SSL de cas.ensiie.fr et aurionweb.ensiie.fr sont auto-signés, Android les refuse
    //
    // Ici on indique à Android qu'on fait confiance à uniquement ces 2 certificats
    public static SSLContext getContextForTrustedCertificates(Context c) throws GeneralSecurityException, IOException {
        CertificateFactory cf = getSSLFactory();
        if(sslContext == null){
            // Fetch certificate from my perso
            Certificate cas = getCasCert(c);
            Certificate aurionweb = getAurionCert(c);

            // Create a KeyStore containing our trusted CAs
            String keyStoreType = KeyStore.getDefaultType();
            KeyStore keyStore = KeyStore.getInstance(keyStoreType);
            keyStore.load(null, null);
            keyStore.setCertificateEntry("cas", cas);
            keyStore.setCertificateEntry("aurionweb", aurionweb);

            // Create a TrustManager that trusts the CAs in our KeyStore
            String tmfAlgorithm = TrustManagerFactory.getDefaultAlgorithm();
            TrustManagerFactory tmf = TrustManagerFactory.getInstance(tmfAlgorithm);
            tmf.init(keyStore);

            // Create an SSLContext that uses our TrustManager
            sslContext = SSLContext.getInstance("TLS");
            sslContext.init(null, tmf.getTrustManagers(), null);
        }

        return sslContext;
    }

    public static Point getScreenSize(Context c){
        WindowManager wm = (WindowManager) c.getSystemService(Context.WINDOW_SERVICE);
        Display display = wm.getDefaultDisplay();
        Point size = new Point();
        display.getSize(size);
        return size;
    }
    public static JsonObject strToJson(String text) {
        JsonObject root = new JsonObject();
        root.addProperty("text", text);
        return root;
    }

    public static String stripAccents(String s) {
        s = Normalizer.normalize(s, Normalizer.Form.NFD);
        s = s.replaceAll("[\\p{InCombiningDiacriticalMarks}]", "");
        return s;
    }

    public static String bytesToString(long bytes) {
        long absB = bytes == Long.MIN_VALUE ? Long.MAX_VALUE : Math.abs(bytes);
        if (absB < 1024) {
            return bytes + " o";
        }
        long value = absB;
        CharacterIterator ci = new StringCharacterIterator("KMGTPE");
        for (int i = 40; i >= 0 && absB > 0xfffccccccccccccL >> i; i -= 10) {
            value >>= 10;
            ci.next();
        }
        value *= Long.signum(bytes);
        return String.format(Locale.FRENCH, "%.1f %co", value / 1024.0, ci.current());
    }
}
