package fr.nitorac.aurionweb.fragments;

import android.content.Context;
import android.net.http.SslError;
import android.os.Build;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.webkit.CookieManager;
import android.webkit.SslErrorHandler;
import android.webkit.WebView;
import android.webkit.WebViewClient;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;

import java.security.cert.Certificate;
import java.security.cert.CertificateException;
import java.util.HashMap;
import java.util.Map;

import fr.nitorac.aurionweb.R;
import fr.nitorac.aurionweb.Utils;
import fr.nitorac.aurionweb.aurionweb.AurionManager;
import fr.nitorac.aurionweb.aurionweb.AurionSession;
import fr.nitorac.aurionweb.aurionweb.CheckConnectionTask;
import fr.nitorac.aurionweb.security.SecurityManager;

public abstract class AurionViewFragment extends Fragment {

    protected AurionSession currentSession = new AurionSession();
    protected WebView webView;
    private View rootView;

    public abstract void preload();

    public void pageFinished(WebView view, String url){

    }

    public void networkInit(AurionSession session){

    }

    protected void initLoad(){
        CheckConnectionTask check = new CheckConnectionTask(getContext(), currentSession, res -> {
            if(res){
                setCookie();
                preload();
            }else{
                AurionManager.getInstance().spawnLoginDialog(getContext(), (ignored) ->{
                    currentSession.clear();
                    initLoad();
                });
            }
        }, this::networkInit);
        check.execute(SecurityManager.creds);
    }

    public View onCreateView(@NonNull LayoutInflater inflater,
                             ViewGroup container, Bundle savedInstanceState) {
        if(rootView == null){
            rootView = inflater.inflate(R.layout.fragment_aurionview, container, false);

            webView = rootView.findViewById(R.id.aurionView);

            webView.getSettings().setSupportZoom(true);
            webView.getSettings().setJavaScriptEnabled(true);

            webView.setWebViewClient(new WebViewClient() {
                @Override
                public void onPageFinished(WebView view, String url) {
                    pageFinished(view, url);
                    if(url.contains("cas.ensiie.fr")){
                        initLoad();
                    }
                    super.onPageFinished(view, url);
                }

                @Override
                public void onReceivedError(WebView view, int errorCode, String description, String failingUrl) {
                    Log.e("WebClient", description);
                }

                @Override
                public void onReceivedSslError(WebView view, SslErrorHandler handler, SslError er) {
                    Context c = AurionViewFragment.this.getContext();
                    try{
                        Certificate cas = Utils.getCasCert(c);
                        Certificate aurion = Utils.getAurionCert(c);
                        handler.proceed();
                    }catch (CertificateException e){
                        e.printStackTrace();
                        Toast.makeText(c, "Erreur de certificat SSL : " + e.getLocalizedMessage(), Toast.LENGTH_LONG).show();
                        handler.cancel();
                    }
                }

            });

            initLoad();

            if(Build.VERSION.SDK_INT > Build.VERSION_CODES.LOLLIPOP) {
                CookieManager.getInstance().setAcceptThirdPartyCookies(webView, true);
                CookieManager.getInstance().removeAllCookies(bool -> {});
            }
        }

        return rootView;
    }

    @Override
    public void onResume() {
        super.onResume();
        setCookie();
    }

    public void setCookie(){
        CookieManager.getInstance().setCookie("https://aurionweb.ensiie.fr", "JSESSIONID=" + currentSession.getJSESSIONID());
    }

    public Map<String, String> getHeaders(){
        HashMap<String, String> res = new HashMap<>();
        res.put("Accept", "*/*");
        res.put("Accept-Encoding", "gzip, deflate");
        res.put("Cache-Control", "no-cache");

        return res;
    }
}
